# SYNOPSIS
#
#   BASSO_CHECK_MEX
#
# DESCRIPTION
#
#   Searches for a functional Matlab mex-compiler that supports C99 and sets variable MEXC if any is found. If none is found
#   Automake conditional BASSO_WITH_MATLAB is set to false.
#   The variable MEXEXT is set to the mex extension of the system and the 
#   variable MEXCFLAGS is set to the CFLAGS required to support C99 with the 
#   compiler.

#serial 1

AC_DEFUN([BASSO_CHECK_MEX],
[AC_ARG_WITH([matlab],
  [AS_HELP_STRING([--with-matlab@<:@=path@:>@],
    [compile the Matlab MEX files (using MEX in path)])],
  [AS_CASE([$withval], [yes], [basso_mex_path=no; basso_with_matlab_given=yes],
                       [no],  [],
		              [with_matlab=yes; basso_mex_path=$withval; basso_with_matlab_given=yes])],
  [with_matlab=yes; basso_mex_path=no])
AC_ARG_VAR([MEXC], [the Matlab MEX compiler])
AC_ARG_VAR([MEXCFLAGS], [the CFLAGS for the MEX compiler; N.B. the MEX compiler must support C99])


if test "x$MEXCFLAGS" = xno -o -z "$MEXCFLAGS"; then
  if test "x$ac_cv_prog_cc_c99" = xno ; then
    # The C compiler doesn't support C99, but maybe MEX will?
    MEXCFLAGS=""
  else
    MEXCFLAGS=$ac_cv_prog_cc_c99
  fi
fi

#basso_working_mex="no"
MEXEXT=""
if test "x$with_matlab" != xno; then
  if test "x$basso_mex_path" = xno; then
    basso_mex_path=$PATH
  fi
  # Check first the more specific names; mex is also installed with LaTeX
  AC_PATH_PROGS([MEXC], [matlab-mex mextool mex.bat mex], [no], [$basso_mex_path])
  # Make sure the mex we found actually works
  if test "x$MEXC" != xno; then
    AC_CACHE_CHECK([whether mex compiler works], [basso_cv_working_mex],
    [basso_cv_working_mex="no"
      #AC_MSG_CHECKING([whether mex compiler works])
      cat > conftest.c <<EOF
#include "mex.h"
void mexFunction(int nlhs, mxArray *plhs[[]], int nrhs, const mxArray *prhs[[]])
{ 
  for (int i=0; i<2; i++) // A C99 feature we are going to need
    ; 
  return; 
}
EOF
      if $MEXC CFLAGS="\$CFLAGS $MEXCFLAGS" -output conftest conftest.c 2>/dev/null 1>&2 
      then
        basso_cv_working_mex="yes"
        # Find the MEXEXT in a brute-force way
        # MEX isn't working if the output isn't one of these
        AS_IF([test -a conftest.mexa64],    [MEXEXT=".mexa64"],
              [test -a conftest.mexmaci64], [MEXEXT=".mexmaci64"],
              [test -a conftest.mexw64],    [MEXEXT=".mexw64"],
              [basso_working_mex="no"])
      fi
      # Clean up; if mex is from tex, it produces texput.log or conftest.log, which we must also clean
      rm -f conftest.c conftest.o conftest.dll conftest.mex* texput.log conftest.log
      #AC_MSG_RESULT([$basso_working_mex])
    ])
    if test $basso_cv_working_mex = yes; then
      # Find the MEXEXT in a brute-force way - test the suffix of the conftest.mex*
      # We have to make a new file, as the previous test might have been skipped due to cached result

      AC_CACHE_CHECK([for mex extension], [basso_cv_MEXEXT],
        [basso_cv_MEXEXT="no"
	cat > conftest.c << EOF
#include "mex.h"
void mexFunction(int nlhs, mxArray *plhs[[]], int nrhs, const mxArray *prhs[[]])
{ 
  for (int i=0; i<2; i++) // A C99 feature we are going to need
    ; 
  return; 
}
EOF
        if $MEXC CFLAGS="\$CFLAGS $MEXCFLAGS" -output conftest conftest.c 2>/dev/null 1>&2 
        then
          # MEX isn't working if the output isn't one of these
          AS_IF([test -a conftest.mexa64],    [basso_cv_MEXEXT=".mexa64"],
                [test -a conftest.mexmaci64], [basso_cv_MEXEXT=".mexmaci64"],
                [test -a conftest.mexw64],    [basso_cv_MEXEXT=".mexw64"],
                [basso_cv_working_mex="no"])
	else
	  basso_cv_working_mex="no"
	  basso_cv_MEXEXT="no" 
        fi
	# Clean up; if mex is from tex, it produces texput.log or conftest.log, which we must also clean
        rm -f conftest.c conftest.o conftest.dll conftest.mex* texput.log conftest.log
      ])
      MEXEXT=$basso_cv_MEXEXT
      # fail if mex was found broken at this part
      if test $basso_cv_working_mex = no; then
        AC_MSG_ERROR([the mex compiler was not working properly])
      fi
    else
      MEXC="no"
      if test "x$basso_with_matlab_given" = xyes; then
        AC_MSG_ERROR([no working mex-compiler found])
      fi
      with_matlab="no"
    fi 
  fi # if test $basso_cv_working_mex = yes; then find MEXEXT
fi #if with_matlab != no


AC_SUBST([MEXEXT])
AM_CONDITIONAL([BASSO_WITH_MATLAB], [test "x$with_matlab" = xyes])
])
