#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

/*
 * We don't seem to need omp.h here - PM
#include <omp.h>
 */

#include "basso_commons.h"
#include "BasicMatrixOperations.h"
#include "MemoryManagement.h"
#include "../../include/AssociationMatrix.h"
#include "../../include/DataTypes.h"


basso_bmatrix_rm *restrict
basso_associationMatrix(const basso_bmatrix_rm *restrict A,
				 		const double threshold)
{
	const uint64_t mRows = A->mRows;

	basso_bmatrix_rm* C = checkedCallocBitMatRowWise(mRows, mRows);

	const uint64_t integerPerRowOfA = A->integerPerRow;
	const uint64_t integerPerRowOfC = C->integerPerRow;

	// we do the computations in chunks of 64x64 blocks of matrix C
	// in order to avoid multiple processors writing to the same bits of C
	for (uint64_t chunkRow = 0; chunkRow < C->integerPerRow; ++chunkRow) {
#pragma omp parallel for
		for (uint64_t chunkCol = 0; chunkCol < C->integerPerRow; ++chunkCol) {
			
			// now compute the actual 64x64 block of C
			for (uint64_t i=chunkRow*64; i < (chunkRow+1)*64 && i < mRows; ++i) {
				// due to the symmetry of C, we only have to compute for j above the diagonal
				uint64_t startingJ = (i > chunkCol*64) ? i : chunkCol*64;

				for (uint64_t j=startingJ; j < (chunkCol+1)*64 && j < mRows; ++j) {

					// compute <A_i, A_j>
					uint64_t dotProd_ij = 0;
					for (uint64_t k=0; k<integerPerRowOfA; ++k) {
						dotProd_ij += POPCNT64( A->matrixRow[i*integerPerRowOfA + k] & A->matrixRow[j*integerPerRowOfA + k] );
					}
					
					// set C[i,j] = (<A_i, A_j> / <A_i, A_i> > threshold)
					// Remark: Pauli's thesis uses >= instead of >.
					//		   Here this is changed in order to stay compatible
					//		   with older verions of asso.
					if ( dotProd_ij > threshold*A->onesPerRow[i]) {
						C->matrixRow[j*integerPerRowOfC + i/64] |= (ONE << i%64);
					}

					// set C[j,i] = (<A_i, A_j> / <A_j, A_j> > threshold)
					// Remark: See remark above.
					if ( dotProd_ij > threshold*A->onesPerRow[j]) {
						C->matrixRow[i*integerPerRowOfC + j/64] |= (ONE << j%64);
					}
				}
			}
		}
	}

	return C;
}

