#ifndef _BASSO_BASICMATRIXOPERATIONS_H
#define _BASSO_BASICMATRIXOPERATIONS_H

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "basso_commons.h"
#include "../../include/DataTypes.h"

basso_bmatrix_cm *restrict
colWiseMatrixFromRowWiseMatrix(const basso_bmatrix_rm *restrict A);

basso_bmatrix_rm *restrict
rowWiseMatrixFromColWiseMatrix(const basso_bmatrix_cm *restrict A);

uint64_t
basso_count_nnz_rm(const basso_bmatrix_rm *restrict A);

uint64_t
basso_count_nnz_cm(const basso_bmatrix_cm *restrict A);

static inline int
basso_get_element_rm(const basso_bmatrix_rm *restrict M,
		     const uint64_t row,
		     const uint64_t col)
{
  const uint64_t integerPerRow = M->integerPerRow;
  uint64_t val;
  val = M->matrixRow[row*integerPerRow + col/64] & (ONE << col%64);
  return (int)(val >> col%64);
}

static inline int
basso_get_element_cm(const basso_bmatrix_cm *restrict M,
		     const uint64_t row,
		     const uint64_t col)
{
  const uint64_t integerPerCol = M->integerPerCol;
  uint64_t val;
  val = M->matrixCol[col*integerPerCol + row/64] & (ONE << row%64);
  return (int)(val >> row%64);
}


static inline void
basso_set_matrix_element_rm(basso_bmatrix_rm *restrict M, const uint64_t row, const uint64_t col, const int value)
{
  const uint64_t integerPerRow = M->integerPerRow;
  const int currVal = basso_get_element_rm(M, row, col);
  if (value && !currVal) {
    M->matrixRow[row*integerPerRow + col/64] |=  (ONE << col%64);
    M->onesPerRow[row]++;
  } else if (!value && currVal) {
    M->matrixRow[row*integerPerRow + col/64] &= ~(ONE << col%64);
    M->onesPerRow[row]--;
  }
}


#endif

