#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "basso_commons.h"
#include "MemoryManagement.h"

void
printCallocFailedError(uint64_t requestedInts)
{
	fprintf(stderr, "Failed to allocate space for %llu uint64_t values. "
					"Original message: %s\n", requestedInts, strerror(errno));
	exit(EXIT_FAILURE);
}

basso_bmatrix_rm*
checkedCallocBitMatRowWise(const size_t mRows, const size_t nCols)
{
	basso_bmatrix_rm* result = malloc(sizeof(basso_bmatrix_rm));
	if (result == NULL) return NULL;
	
	result->mRows = mRows;
	result->nCols = nCols;

	result->onesPerRow = checkedCallocUInt64_t(mRows);
	if (result->onesPerRow == NULL) {
	  free(result);
	  return NULL;
	}
	
	result->integerPerRow = nCols/64 + (nCols%64 > 0 ? 1 : 0);

	result->matrixRow = checkedCallocUInt64_t(mRows * result->integerPerRow);
	if (result->matrixRow == NULL) {
	  free(result->onesPerRow);
	  free(result);
	  return NULL;
	}

	return result;
}

basso_bmatrix_cm*
checkedCallocBitMatColWise(const size_t mRows, const size_t nCols)
{
	basso_bmatrix_cm* result = malloc(sizeof(basso_bmatrix_cm));
	if (result == NULL) return NULL;

	result->mRows = mRows;
	result->nCols = nCols;

	result->integerPerCol = mRows/64 + (mRows%64 > 0 ? 1 : 0);

	result->matrixCol = checkedCallocUInt64_t(nCols * result->integerPerCol);
	if (result->matrixCol == NULL) {
	  free(result);
	  return NULL;
	}

	return result;
}

/**
 * Returns zeroed allocated memory for a pointer to numberOfElements uint64_t's.
 */
uint64_t*
checkedCallocUInt64_t(const size_t numberOfElements)
{
	uint64_t* result = calloc(numberOfElements, sizeof(uint64_t));
	/*
	 * This error checking should be done in the calling function - Pauli
	 *
	if (result == NULL) {
		fprintf(stderr, "Failed to allocate space for %zu uint64_t values. "
						"Original message: %s\n", numberOfElements, strerror(errno));
		exit(EXIT_FAILURE);
	}
	*/

	return result;
}

void
freeBitMatRowWise(basso_bmatrix_rm* bitMat)
{
  if (bitMat != NULL) {
	free(bitMat->onesPerRow);
	free(bitMat->matrixRow);
  }
  free(bitMat);
}

void
freeBitMatColWise(basso_bmatrix_cm* bitMat)
{
  if (bitMat != NULL) free(bitMat->matrixCol);
  free(bitMat);
}

