/* 
 * basso_strto.c
 *
 * Helper functions to convert strings to number or exit in failure
 *
 */

#include "basso_strto.h"
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>



uint64_t
strtoullOrExit(const char *restrict str, const char *restrict what)
{
      char *tailptr;
      int64_t out; // We use signed integers to catch negative values
      errno = 0;

#ifdef HAVE_STRTOLL
      out = strtoll(str, &tailptr, 10);
      if (tailptr[0] != '\0') {
	fprintf(stderr, "Error when setting the %s: Cannot convert string \"%s\" to an unsigned integer\n", what, tailptr);
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }
#else
#  ifdef HAVE_ATOLL
      out = atoll(str);
#  else
#    error "Your system does not support conversions from strings to 64-bit integers"
#  endif /* HAVE_ATOLL */
#endif /* HAVE_STRTOULL */
      if (errno != 0) {
	fprintf(stderr, "Error when setting the %s: %s\n", what, strerror(errno));
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }

      if (out < 0) {
	fprintf(stderr, "Error when setting the %s: %s is a negative number\n",
		what, str);
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }

      return out;
}

int
strtolOrExit(const char *restrict str, const char *restrict what)
{
      char *tailptr;
      int out;
      errno = 0;

#ifdef HAVE_STRTOL
      out = strtol(str, &tailptr, 10);
      if (tailptr[0] != '\0') {
	fprintf(stderr, "Error when setting the %s: Cannot convert string \"%s\" to an integer\n", what, tailptr);
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }
#else
#  ifdef HAVE_ATOI
      out = atoi(str);
#  else
#    error "Your system does not support conversions from strings to integers"
#  endif /* HAVE_ATOI */
#endif /* HAVE_STRTOL */

      if (errno != 0) {
	fprintf(stderr, "Error when setting the %s: %s\n", what, strerror(errno));
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }
      return out;
}

double
strtodOrExit(const char *restrict str, const char *restrict what)
{
      char *tailptr;
      double out;
      errno = 0;

#ifdef HAVE_STRTOD
      out = strtod(str, &tailptr);
      if (tailptr[0] != '\0') {
	fprintf(stderr, "Error when setting the %s: Cannot convert string \"%s\" to double\n", what, tailptr);
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }
#else
#  ifdef HAVE_ATOF
      out = atof(str);
#  else
#    error "Your system does not support conversion of strings to floats"
#  endif /* HAVE_ATOF */
#endif /* HAVE_STRTOD*/

      if (errno != 0) {
	fprintf(stderr, "Error when setting the %s: %s\n", what, strerror(errno));
	exit(BASSO_EXIT_INVALID_PARAMETER);
      }
      return out;
}
