/*
 * Function definitions and macros for command-line interface of Basso
 */

#ifndef _BASSO_CMDLINE_H
#define _BASSO_CMDLINE_H

/* We require POSIX 1995 / X/Open 5 */
#define _GNU_SOURCE
// #define _XOPEN_SOURCE 500

#include "DataTypes.h"
                                            // sysexits.h names
#define BASSO_EXIT_SUCCESS                0
#define BASSO_EXIT_UNKNOWN_ERROR          1
#define BASSO_EXIT_UNIMPLEMENTED         70 // EX_SOFTWARE
#define BASSO_EXIT_INVALID_PARAMETER     64 // EX_USAGE
#define BASSO_EXIT_INVALID_INPUT         65 // EX_DATAERR
#define BASSO_EXIT_INVALID_OUTPUT        73 // EX_CANTCREAT
#define BASSO_EXIT_MEMORY                71 // EX_OSERR
#define BASSO_EXIT_IOERROR               74 // EX_IOERR

#define BASSO_RETURN_SUCCESS              0
#define BASSO_RETURN_IOFAIL              10
#define BASSO_RETURN_IOPERM              15
#define BASSO_RETURN_INVALID_INPUT       20

/* 
 * Read and write MatrixMarket and binary files 
 */

typedef struct basso_bbm_header {
           char magic[6];
  unsigned char reserved;
  unsigned char flags;
  uint64_t rows;
  uint64_t cols;
} basso_bbm_header;

basso_bmatrix_rm *
read_mm_matrix(const char *file);

basso_bmatrix_rm *
read_bbm_matrix(const char *file);

int
write_mm_matrix_rm(const char *restrict filename, 
		   const basso_bmatrix_rm *restrict M,
		   const char *restrict comments);

int
write_mm_matrix_cm(const char *restrict filename, 
		   const basso_bmatrix_cm *restrict M,
		   const char *restrict comments);

int
write_bbm_matrix_rm(const char *restrict filename,
                    const basso_bmatrix_rm *restrict M);

int
write_bbm_matrix_cm(const char *restrict filename,
                    const basso_bmatrix_cm *restrict M);


/*
 * Macros to check for supported file suffices 
 */
#define BASSO_BINARY_SUFFIX "bbm"
#define BASSO_MM_SUFFIX "mtx"
#define is_suffix_matrixmarket(suffix) (!strcmp((suffix), "mm") || !strcmp((suffix), "mmc") || !strcmp((suffix), BASSO_MM_SUFFIX))
#define is_suffix_bassobinary(suffix) (!strcmp((suffix), BASSO_BINARY_SUFFIX))

/*
 * Enums and structs for internal use
 */

enum factor_side {
  LEFT,
  RIGHT
};



#endif /* _BASSO_CMDLINE_H */
